#  $Id: tsfile.py 40 2007-08-06 11:18:04Z ogre.crewman $
#
#  This file is part of the UT3 project
#
#  Copyright (C) 2007 Marco Wegner <ogre.crewman@googlemail.com>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; see the file COPYING.  If not, write to
#  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#  Boston, MA 02110-1301, USA.


"""
This module contains classes and functions necessary for parsing and writing
Qt's TS files.
"""

__version__ = "$Revision: 40 $"


# -----------------------------------------------------------------------------
# Import statements
# -----------------------------------------------------------------------------

# modules
import os, sys, xml.dom
from xml.dom import minidom
from ut3 import store, support
# classes
from ut3.store import UT3Item, UT3Store
# functions
from xml.dom.ext import PrettyPrint


# -----------------------------------------------------------------------------
# Classes
# -----------------------------------------------------------------------------

class Ut3TsFile:
    """
    This class represents a Qt TS file.

    @author: Marco Wegner
    """

    # -------------------------------------------------------------------------
    # Public methods
    # -------------------------------------------------------------------------

    def __init__(self, name):
        """
        Creates a new TS file instance.

        @param name: The TS file name or None for writing the file to stdout.
        """

        self.__name = name

        return

    # -------------------------------------------------------------------------

    def parse(self):
        """
        Parse the TS file.

        @return: A translation entry store containing the parsed entries.
        """

        # check if the PO/POT file is readable
        if self.__name is None or not os.access(self.__name, os.R_OK):
            support.print_error_message("TS file does not exist or is not readable")
            sys.exit( )

        doc = minidom.parse(self.__name).documentElement

        tstore = UT3Store( )

        context_elems = doc.getElementsByTagName('context')
        entry_list = _handle_context_elements(context_elems)
        tstore.append_entries(entry_list)

        doc.unlink( )

        return tstore

    # -------------------------------------------------------------------------

    def write(self, tstore):
        """
        Writes a Qt TS file.

        @param tstore: The UT3 translation store with the items to be written.
        """

        if not isinstance(tstore, UT3Store):
            raise TypeError

        if self.__name is not None:
            # check if the file is writable
            if not os.access(os.path.dirname(self.__name), os.W_OK):
                support.print_error_message("PO file not writable")
                sys.exit( )
            out = file(self.__name, 'w')
        else:
            out = sys.stdout

        impl = xml.dom.getDOMImplementation( )
        doctype = impl.createDocumentType('TS', None, None)
        doc = impl.createDocument(None, "TS", doctype)
        root = doc.documentElement

        source_list = tstore.get_sources( )
        if len(source_list) == 0:
            context_entries = tstore.get_by_source("")
            root.appendChild(_generate_context_element(doc, "", context_entries))
        else:
            for context_name in source_list:
                context_entries = tstore.get_by_source(context_name)
                root.appendChild(_generate_context_element(doc, context_name, context_entries))

        PrettyPrint(root, out)

        if not self.__name is None:
            out.close( )

        doc.unlink( )

        return


# -----------------------------------------------------------------------------
# Helper functions
# -----------------------------------------------------------------------------

def _handle_context_elements(elem_list):
    """
    Creates a list of UT3 translation items from a list of context elements.

    @param elem_list: The list of context elements.
    @return: The list of generated UT3 translation items.
    """

    entry_list = []
    for elem in elem_list:
        name_elem = elem.getElementsByTagName('name')[0]
        name = _handle_name_element(name_elem)
        message_elems = elem.getElementsByTagName('message')
        entry_list.extend(_handle_message_elements(message_elems, name))
    return entry_list

# -----------------------------------------------------------------------------

def _handle_name_element(elem):
    """
    Extracts the context name from a name element.

    @param elem: The name element.
    @return: The extracted context name.
    """

    return _get_node_text(elem.childNodes)

# -----------------------------------------------------------------------------

def _handle_message_elements(elem_list, context):
    """
    Creates a list of UT3 translation items from a list of message elements
    belonging to the specified context.

    @param elem_list: The list of message elements.
    @param context: The context of the message elements.
    @return: The list of generated UT3 translation items.
    """

    entry_list = []
    for elem in elem_list:
        entry = _make_entry(elem, context)
        entry_list.append(entry)
    return entry_list

# -----------------------------------------------------------------------------

def _make_entry(message, context):
    """
    Creates a UT3 translation item from a message element and the context this
    message element belongs to.

    @param message: The message element.
    @param context: The context of the message element.
    @return: The generated UT3 translation item.
    """

    source = _extract_source(message)
    entry = UT3Item(source)
    translation, status = _extract_translation(message)
    entry.translation = translation
    entry.status = status
    entry.comment = _extract_comment(message)
    entry.add_source(context)
    return entry

# -----------------------------------------------------------------------------

def _extract_source(message):
    """
    Extracts a source message from a message element.

    @param message: The current message element.
    @return: The extracted source message text.
    """

    elem = message.getElementsByTagName('source')[0]
    return _get_node_text(elem.childNodes)

# -----------------------------------------------------------------------------

def _extract_translation(message):
    """
    Extracts a translation from a message element.

    @param message: The current message element.
    @return: The extracted translation text and the status of the translation.
    """

    elem = message.getElementsByTagName('translation')[0]
    status = store.STATUS_COMPLETE
    if elem.hasAttribute('type') and elem.getAttribute('type') == "unfinished":
        status = store.STATUS_UNFINISHED
    translation = _get_node_text(elem.childNodes)
    return translation, status

# -----------------------------------------------------------------------------

def _extract_comment(message):
    """
    Extracts a comment from a message element.

    @param message: The current message element.
    @return: The extracted comment text.
    """

    comment = None
    elem_list = message.getElementsByTagName('comment')
    if len(elem_list) > 0:
        elem = elem_list[0]
        comment = _get_node_text(elem.childNodes)
    return comment

# -----------------------------------------------------------------------------

def _get_node_text(node_list):
    """
    Extracts the complete node text from a list of nodes.

    @param node_list: The list of nodes.
    @return: The assembled text.
    """

    text = ""
    for node in node_list:
        if node.nodeType == node.TEXT_NODE:
            text = text + node.data
    return text

# -----------------------------------------------------------------------------

def _generate_context_element(doc, source, entries):
    """
    Generates a context element.

    @param doc: The DOM document.
    @param source: The actual source file name.
    @param entries: The list of UT3 translation items in this context.
    @return: The generated element.
    """

    elem = doc.createElement('context')
    elem.appendChild(_generate_name_element(doc, source))
    for ent in entries:
        elem.appendChild(_generate_message_element(doc, source, ent))

    return elem

# -----------------------------------------------------------------------------

def _generate_name_element(doc, source):
    """
    Generates a name element. This element represents the name of the context.

    @param doc: The DOM document.
    @param source: The actual source file name.
    @return: The generated element.
    """

    elem = doc.createElement('name')
    if source is not None and source != "":
        elem.appendChild(doc.createTextNode(source))

    return elem

# -----------------------------------------------------------------------------

def _generate_message_element(doc, source, ent):
    """
    Generates a message element.

    @param doc: The DOM document.
    @param source: The actual source file name.
    @param ent: The current UT3 translation item where to create the message
        element from.
    @return: The generated element.
    """

    elem = doc.createElement('message')

    elem.appendChild(_generate_location_element(doc, source))
    elem.appendChild(_generate_source_element(doc, ent.message))
    elem.appendChild(_generate_translation_element(doc, ent.translation, ent.status))
    if ent.comment is not None:
        elem.appendChild(_generate_comment_element(doc, ent.comment))

    return elem

# -----------------------------------------------------------------------------

def _generate_location_element(doc, source):
    """
    Generates a location element.

    @param doc: The DOM document.
    @param source: The actual source file name.
    @return: The generated element.
    """

    elem = doc.createElement('location')
    elem.setAttribute('filename', source)
    elem.setAttribute('line', str(0))

    return elem

# -----------------------------------------------------------------------------

def _generate_source_element(doc, message):
    """
    Generates a source element.

    @param doc: The DOM document.
    @param message: The actual source message text.
    @return: The generated element.
    """

    elem = doc.createElement('source')
    elem.appendChild(doc.createTextNode(message))

    return elem

# -----------------------------------------------------------------------------

def _generate_translation_element(doc, translation, status):
    """
    Generates a translation element.

    @param doc: The DOM document.
    @param translation: The actual translation text.
    @param status: The status of the translation.
    @return: The generated element.
    """

    elem = doc.createElement('translation')

    if translation is not None and translation != "":
        elem.appendChild(doc.createTextNode(translation))

    if status == store.STATUS_UNFINISHED:
        elem.setAttribute('type', 'unfinished')

    return elem

# -----------------------------------------------------------------------------

def _generate_comment_element(doc, comment):
    """
    Generates a comment element.

    @param doc: The DOM document.
    @param comment: The actual comment text.
    @return: The generated element.
    """

    elem = doc.createElement('comment')
    elem.appendChild(doc.createTextNode(comment))

    return elem



#  $Id: support.py 36 2007-08-05 15:03:34Z ogre.crewman $
#
#  This file is part of the UT3 project
#
#  Copyright (C) 2007 Marco Wegner <ogre.crewman@googlemail.com>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; see the file COPYING.  If not, write to
#  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#  Boston, MA 02110-1301, USA.


"""
This module contains support functions.
"""

__version__ = "$Revision: 36 $"


# modules
import sys
# functions
from optparse import make_option


# -----------------------------------------------------------------------------
# Global functions
# -----------------------------------------------------------------------------

def get_global_options( ):
    """
    Creates a list of global options (those common to all scripts).

    @return: The option list.
    """

    option_list = [
       make_option("-q", "--quiet", action="store_false", dest="verbose", default=True, help="don't print status messages"),
       make_option("-d", "--delimiter", dest="delim", help="specify the delimiter string"),
       make_option("-l", "--left-delim", dest="ldelim", help="specify the left delimiter string"),
       make_option("-r", "--right-delim", dest="rdelim", help="specify the right delimiter string"),
       make_option("-c", "--comment-delimiter", dest="commdelim", help="specify the delimiter for comments"),
       make_option("-s", "--start-comment", dest="startcomm", help="specify the left delimiter for comments"),
       make_option("-e", "--end-comment", dest="endcomm", help="specify the left delimiter for comments")
   ]

    return option_list

# -----------------------------------------------------------------------------

def print_error_message(message):
    """
    Print an error message in a standardized way.

    @param message: The actual error message.
    """

    sys.stderr.write("WARNING: %s\n" % (message))

    return

# -----------------------------------------------------------------------------

def print_warning_message(message):
    """
    Print a warning message in a standardized way.

    @param message: The actual warning message.
    """

    sys.stderr.write("WARNING: %s\n" % (message))

    return

# -----------------------------------------------------------------------------

def extract_delimiters(options):
    """
    Checks the parser options for sufficient definition of delimiters and then
    extracts these delimiters.

    @param options: The options object as created by the OptionParser.
    @return: A dictionary containing the extracted delimiters.
    """

    delim_dict = {}

    _extract_string_delimiters(options, delim_dict)
    _extract_comment_delimiters(options, delim_dict)

    return delim_dict

# -----------------------------------------------------------------------------

def _extract_string_delimiters(options, delim_dict):
    """
    Extracts the delimiters for translatable strings.

    @param options: The instance containing the options.
    @param delim_dict: The current dictionary for adding the new delimiters.
    """

    # check if any delimiters are defined
    if options.delim is None and options.ldelim is None and options.rdelim is None:
        print_error_message("no delimiters defined!")
        sys.exit( )

    # check for conflicting delimiter options
    if options.delim is not None:
        # make sure that neither ldelim nor rdelim are specified if delim
        # is specified at the same time
        if options.ldelim is not None or options.rdelim is not None:
            print_error_message("delim and ldelim/rdelim can't be " \
                + "specified at the same time!")
            sys.exit( )
        delim_dict['ldelim'] = options.delim
        delim_dict['rdelim'] = options.delim
    else:
        # if delim is not defined then make sure that both ldelim and relim
        # are defined
        if options.ldelim is None or options.rdelim is None:
            print_error_message("both ldelim and rdelim must be specified!")
            sys.exit( )
        delim_dict['ldelim'] = options.ldelim
        delim_dict['rdelim'] = options.rdelim

    return

# -----------------------------------------------------------------------------

def _extract_comment_delimiters(options, delim_dict):
    """
    Extracts the delimiters for comments/hints.

    @param options: The instance containing the options.
    @param delim_dict: The current dictionary for adding the new delimiters.
    """

    if options.commdelim is None and options.startcomm is None and options.endcomm is None:
        # no hint/comment delimiters defined
        return

    if options.commdelim is not None:
        if options.startcomm is not None or options.endcomm is not None:
            print_error_message("commdelim and startcomm/endcomm can't be " \
                + "specified at the same time!")
            sys.exit( )
        delim_dict['comm_ldelim'] = options.commdelim
        delim_dict['comm_rdelim'] = options.commdelim
    else:
        if (options.startcomm is not None and options.endcomm is None) or (options.startcomm is None and options.endcomm is not None):
            print_error_message("both start and end delimiters must be defined for comments")
            sys.exit( )
        delim_dict['comm_ldelim'] = options.startcomm
        delim_dict['comm_rdelim'] = options.endcomm

    return



#! /usr/bin/env python

#  $Id: template2pot.py 37 2007-08-05 21:30:24Z ogre.crewman $
#
#  This file is part of the UT3 project
#
#  Copyright (C) 2007 Marco Wegner <ogre.crewman@googlemail.com>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; see the file COPYING.  If not, write to
#  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#  Boston, MA 02110-1301, USA.


__version__ = "$Revision: 37 $"


# -----------------------------------------------------------------------------
# Import declarations
# -----------------------------------------------------------------------------

# modules
import os, sys
from ut3 import support
# classes
from optparse import OptionParser
from ut3.pofile import Ut3PoFile
from ut3.store import UT3Store
from ut3.template import UT3Template
# functions
from optparse import make_option


# -----------------------------------------------------------------------------
# Global functions
# -----------------------------------------------------------------------------

def main( ):
    """
    The main function.

    @author: Marco Wegner
    """

    usage = "usage: %prog [options] template1 [template2, ...]"
    parser = OptionParser(usage, conflict_handler="resolve")
    parser.add_options(support.get_global_options( ))
    parser.add_options(_get_local_options( ))

    (options, args) = parser.parse_args( )

    # the script has been called without any options or arguments
    if len(sys.argv) == 1:
        parser.print_help( )
        sys.exit( )

    delimiters = support.extract_delimiters(options)

    # check if the template file is defined
    if len(args) == 0:
        support.print_error_message("no template file(s) specified")
        sys.exit( )

    if options.verbose:
        print "\nExtracting translatable strings from the template files:"

    super_store = UT3Store( )

    count = 0
    for tmpl_file in args:
        tmpl = UT3Template(tmpl_file, delimiters)
        tmpl_store = tmpl.parse( )
        num_rem = len(tmpl_store.get_all( ))
        count = count + num_rem

        super_store.extend(tmpl_store)

        if options.verbose:
            tname = os.path.basename(tmpl_file)
            print " * " + tname + ": " + str(num_rem) + " messages"

    if options.verbose:
        num_rem = len(super_store.get_all( ))
        print "\nHandling duplicates: removing %i strings" % (count - num_rem)
        print "\nRemaining to be written: %i strings" % (num_rem)
        print "\nWriting the template...\n"

    pofile = Ut3PoFile(options.outfile)
    pofile.write(super_store, True)

    if options.verbose:
        print "\nDone."

    return


# -----------------------------------------------------------------------------
# Helper functions
# -----------------------------------------------------------------------------

def _get_local_options( ):
    """
    Returns a list of options which are only used in this script.

    @return: The option list.
    """

    option_list = [
       make_option("-o", "--output", dest="outfile", help="specify the output file")
    ]

    return option_list

# -----------------------------------------------------------------------------

if __name__ == "__main__":
    main( )



#  $Id: template.py 36 2007-08-05 15:03:34Z ogre.crewman $
#
#  This file is part of the UT3 project
#
#  Copyright (C) 2007 Marco Wegner <ogre.crewman@googlemail.com>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; see the file COPYING.  If not, write to
#  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#  Boston, MA 02110-1301, USA.


"""
This module contains the classes and functions necessary for dealing with
UT3 template files.
"""

__version__ = "$Revision: 36 $"


# modules
import logging, os, re, sys
# classes
from ut3.store import UT3Item, UT3Store
# functions
from ut3.support import print_error_message


# -----------------------------------------------------------------------------
# Classes
# -----------------------------------------------------------------------------

class UT3Template:
    """
    This class represents a UT3 template file.

    @author: Marco Wegner
    """


    # -------------------------------------------------------------------------
    # Public methods
    # -------------------------------------------------------------------------

    def __init__(self, name, delims):
        """
        Creates a new UT3 template file instance.

        @param name: The template file name.
        @param delims: The delimiters options as a dictionary.
        """

        # check if the template is readable
        if name is None or not os.access(name, os.R_OK):
            print_error_message("specified template file does not exist " \
                + "or is not readable")
            sys.exit( )

        self.__name = name
        self.__delims = delims

        return

    # -------------------------------------------------------------------------

    def parse(self):
        """
        Parses the template file and extracts a list of UT3 translation items
        as an UT3 translation store.

        @return: The translation store filled with the parsed entries.
        """

        tbase = os.path.basename(self.__name)

        ldelim = self.__delims['ldelim']
        rdelim = self.__delims['rdelim']

        pattern = _assemble_split_pattern(ldelim, rdelim)

        tstore = UT3Store( )

        inside = False    #  whether or not we're inside a translatable
        trstr = ""        #  the current translatable string

        for curl in self._get_file_lines( ):
            curl = curl.rstrip('\n')
            slist = re.split(pattern, curl)
            for item in slist:
                if item == ldelim:
                    # reset the translatable string
                    trstr = ""
                    # translatable: begin
                    inside = True
                elif item == rdelim:
                    # extract the comment from the translatable string
                    (trstr, comment) = self._extract_hint(trstr.strip( ))
                    # create new translation entry for this string
                    entry = UT3Item(trstr)
                    entry.add_source(tbase)
                    entry.comment = comment
                    tstore.append(entry)
                    # translatable: end
                    inside = False
                else:
                    # the current string is no delimiter, so it must either
                    # be a translatable (if inside) or not
                    if inside:
                        trstr = trstr + item + " "

        return tstore

    # -------------------------------------------------------------------------

    def convert(self, tstore, outfile):
        """
        Assembles a file from the original template file and a list of
        translations.

        @param tstore: The UT3 translation store.
        @param outfile: The output file name or None for writing to stdout.
        @return: The total number of applied messages and the number of
            untranslated messages for the specified template.
        """

        if not isinstance(tstore, UT3Store):
            raise TypeError

        lines = self._get_file_lines( )

        ldelim = self.__delims['ldelim']
        rdelim = self.__delims['rdelim']

        pattern = _assemble_split_pattern(ldelim, rdelim)

        trlines = []      #  the list keeping all the assembled and
                          #+ translated lines
        inside = False    #  whether or not we're inside a translatable
        trstr = ""        #  the current translatable string
        assembled = ""    #  the current assembled line from the HTML file
        num_msg = 0       #  the total number of applied messages
        num_untr = 0      #  the number of untranslated message used for
                          #+ the current file

        for curl in lines:
            curl = curl.rstrip('\n')
            slist = re.split(pattern, curl)
            for item in slist:
                if item == ldelim:
                    # reset the translatable string
                    trstr = ""
                    # translatable: begin
                    inside = True
                elif item == rdelim:
                    # strip the comment from the translatable string but
                    # ignore it
                    trstr = self._extract_hint(trstr.strip( ))[0]

                    entry = tstore.get(trstr)
                    if entry is not None and entry.translation is not None and entry.translation != "":
                        # we found the correct message and it is even
                        # translated :)
                        trlate = entry.translation
                    else:
                        # there was either no such entry or the entry was not
                        # translated
                        trlate = trstr
                        num_untr = num_untr + 1

                    assembled = assembled + trlate
                    inside = False
                    num_msg = num_msg + 1
                else:
                    if inside:
                        trstr = trstr + item + " "
                    else:
                        assembled = assembled + item

            if not inside:
                trlines.append(assembled)
                assembled = ""

        _write(trlines, outfile)

        return num_msg, num_untr


    # -------------------------------------------------------------------------
    # Helper methods
    # -------------------------------------------------------------------------

    def _get_file_lines(self):
        """
        Returns the list of lines read from a template file.

        @return: The list of lines.
        """

        # read the template file
        tmpl = file(self.__name, 'r')
        lines = tmpl.readlines( )
        tmpl.close( )

        msg = "read %i lines from the template file" % (len(lines))
        logging.info(msg)

        return lines

    # -------------------------------------------------------------------------

    def _extract_hint(self, trstr):
        """
        Extracts a hint/comment from an already extracted translatable string.
        The hint is optional and does not have to be specified at all. If it
        specified it must be located at the start of the translatable string.

        @param trstr: The complete translatable string which contain a hint
            (or possibly not).
        @return: Both the translatable string stripped of the hint and the
            hint itself as a tuple.
        """

        hint = None

        if self.__delims.has_key('comm_ldelim') and self.__delims.has_key('comm_rdelim'):

            ldelim = self.__delims['comm_ldelim']
            rdelim = self.__delims['comm_rdelim']

            if trstr.find(ldelim) == 0:
                epos = trstr.find(rdelim)

                hint = trstr[len(ldelim):epos].strip( )
                trstr = trstr[epos+len(rdelim):].strip( )

        return (trstr, hint)


# -----------------------------------------------------------------------------
# Helper functions
# -----------------------------------------------------------------------------

def _write(lines, outfile):
    """
    Writes a re-assembled file. This file is basically the original template
    with replaced (translated) strings.

    @param lines: The re-assembled (translated) lines from the original
        template.
    @param outfile: The output file name or None for writing to stdout.
    """

    if outfile is not None:
        # check if the file is writable
        if not os.access(os.path.dirname(outfile), os.W_OK):
            print_error_message("PO file not writable")
            sys.exit( )
        out = file(outfile, 'w')
    else:
        out = sys.stdout

    for curl in lines:
        out.write(curl + '\n')

    if not outfile is None:
        out.close( )

    return

# -----------------------------------------------------------------------------

def _assemble_split_pattern(left, right):
    """
    Assembles the regular expression pattern for splitting strings with
    delimiters.

    @param left: The left delimiter.
    @param right: The right delimiter.
    @return: The assembled regular expression pattern.
    """

    if left == right:
        pattern = left
    else:
        pattern = "(%s|%s)" % (left, right)

    logging.debug("message split pattern assembled: %s" % (pattern))

    return pattern



#  $Id: store.py 39 2007-08-06 10:48:59Z ogre.crewman $
#
#  This file is part of the UT3 project
#
#  Copyright (C) 2007 Marco Wegner <ogre.crewman@googlemail.com>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; see the file COPYING.  If not, write to
#  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#  Boston, MA 02110-1301, USA.


"""
This module contains the classes necessary for intermediate storing of
translation items and their properties.
"""

__version__ = "$Revision: 39 $"


# modules
import types


# -----------------------------------------------------------------------------
# Global constants
# -----------------------------------------------------------------------------

STATUS_COMPLETE   = 0
STATUS_UNFINISHED = 1

# -----------------------------------------------------------------------------
# Classes
# -----------------------------------------------------------------------------

class UT3Item:
    """
    This class represents a single translation item.

    @author: Marco Wegner
    """

    STATUS_COMPLETE = 0
    STATUS_UNFINISHED = 1

    def __init__(self, msg):
        """
        Creates a new translation item.

        @param msg: The translatable string.
        """

        self.message = msg
        self.translation = None
        self.status = STATUS_UNFINISHED
        self.hint = None
        self.__sources = set( )

        return

    # -------------------------------------------------------------------------

    def __setattr__(self, name, value):
        """
        Checks types and values for the attributes for validity.

        @param name: The attribute's name.
        @param value: The attribute's value.
        """

        if name == 'message' and (value is None or type(value) not in types.StringTypes):
            raise TypeError
        if name == 'translation' and not (value is None or type(value) in types.StringTypes):
            raise TypeError
        if name == 'comment' and not (value is None or type(value) in types.StringTypes):
            raise TypeError
        if name == 'status' and not (value == STATUS_COMPLETE or value == STATUS_UNFINISHED):
            raise ValueError
        self.__dict__[name] = value

        return

    # -------------------------------------------------------------------------

    def __eq__(self, other):
        """
        Compares this entry and the specified other for equality. Two UT3
        items are considered equal if their (original) translatable strings
        are the same.

        @param other: The other object to compare.
        @return: True if both objects are UT3 items and have the same message
            string, otherwise False.
        """

        return isinstance(other, UT3Item) and self.message == other.message

    # -------------------------------------------------------------------------

    def add_source(self, source):
        """
        Adds a single source file name to the current list of this entry's
        source files. The source is not added if it is already contained in
        the list of sources.

        @param source: The source to add.
        """

        if source is not None:
            self.__sources.add(source.strip( ))
        return

    # -------------------------------------------------------------------------

    def extend_sources(self, source_list):
        """
        Extends the list of the entry's source file names by the specified
        list of sources. Duplicate entries are removed from the resulting
        list.

        @param source_list: The list of sources
        """

        if source_list is not None:
            for source in source_list:
                self.add_source(source)
            
        return

    # -------------------------------------------------------------------------

    def get_sources(self):
        """
        Return a list of the entry's source file names.

        @return: The list of source files.
        """

        return self.__sources

    # -------------------------------------------------------------------------

    def has_source(self, source):
        """
        Checks if the current entry has the specified source among its list of
        source file names.

        @param source: The source file name.
        @return: True if the source is among the entry's sources, else False.
        """

        return source.strip( ) in self.get_sources( )

# -----------------------------------------------------------------------------

class UT3Store:
    """
    This class represents a store of UT3 translation entries.

    @author: Marco Wegner
    """

    def __init__(self):
        """
        Creates a new UT3 translation store.
        """

        self.__entries = []
        self.__sources = set( )

        return

    # -------------------------------------------------------------------------

    def extend(self, other):
        """
        Extend the currenty list of entries by the entries from another store.

        @param other: The other UT3 translation store.
        """

        if not isinstance(other, UT3Store):
            raise TypeError

        self.append_entries(other.get_all( ))

        return

    # -------------------------------------------------------------------------

    def append(self, entry):
        """
        Updates or adds a single entry. If no entry with the same message
        exists in the list, then the complete entry is added to the list.
        Otherwise only the source file names and the hint are updated.

        @param entry: A translatable entry.
        """

        if not isinstance(entry, UT3Item):
            raise TypeError

        contained = False
        for ent in self.__entries:
            if ent == entry:
                sources = entry.get_sources( )
                ent.extend_sources(sources)
                self.__sources = self.__sources.union(set(sources))
                # only copy the translation over if ones wasn't set yet
                if ent.translation == None:
                    ent.translation = entry.translation
                # only copy the comment over if ones wasn't set yet
                if ent.comment == None:
                    ent.comment = entry.comment
                contained = True
                break

        if not contained:
            self.__entries.append(entry)
            self.__sources = self.__sources.union(entry.get_sources( ))

        return

    # -------------------------------------------------------------------------

    def append_entries(self, entry_list):
        """
        Appends a list of translation entries.

        @param entry_list: The list of entries.
        """

        for entry in entry_list:
            self.append(entry)

        return

    # -------------------------------------------------------------------------

    def get_sources(self):
        """
        Return a list of source file names for all the currently stored
        entries.

        @return: The list of source names.
        """

        return self.__sources

    # -------------------------------------------------------------------------

    def get(self, message):
        """
        Return a single entry with the specified original message.

        @param message: The original (translatable) string.
        @return: The found entry or None if no such entry exists in the store.
        """

        entry = None
        for cur_ent in self.__entries:
            if cur_ent.message == message:
                entry = cur_ent
                break
        return entry

    # -------------------------------------------------------------------------

    def get_all(self):
        """
        Return a list of all entries currently in the store.

        @return: The list of all entries.
        """

        return self.__entries

    # -------------------------------------------------------------------------

    def get_unfinished(self):
        """
        Return all unfinished entries.

        @return: The list of unfinished entries.
        """

        return [ent for ent in self.__entries if ent.status == STATUS_UNFINISHED]

    # -------------------------------------------------------------------------

    def get_by_source(self, source):
        """
        Return all entries which have the specified source among their list
        of sources.

        @param source: The source name.
        @return: The list of entries.
        """
        
        if source == "":
            return [ent for ent in self.__entries if len(ent.get_sources( )) == 0]

        return [ent for ent in self.__entries if ent.has_source(source)]

    # -------------------------------------------------------------------------

    def get_unfinished_by_source(self, source):
        """
        Return all unfinished entries which have the specified source among
        their list of sources.

        @param source: The source name.
        @return: The list of unfinished entries.
        """

        if source == "":
            return [ent for ent in self.get_unfinished( ) if len(ent.get_sources( )) == 0]

        return [ent for ent in self.get_unfinished( ) if ent.has_source(source)]



#! /usr/bin/env python

#  $Id: po2final.py 36 2007-08-05 15:03:34Z ogre.crewman $
#
#  This file is part of the UT3 project
#
#  Copyright (C) 2007 Marco Wegner <ogre.crewman@googlemail.com>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; see the file COPYING.  If not, write to
#  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#  Boston, MA 02110-1301, USA.


__version__ = "$Revision: 36 $"


# -------------------------------------------------------------------------
# Import declarations
# -------------------------------------------------------------------------

# modules
import os, sys
from ut3 import support
# classes
from optparse import OptionParser
from ut3.pofile import Ut3PoFile
from ut3.template import UT3Template
# functions
from optparse import make_option


# -----------------------------------------------------------------------------
# Global functions
# -----------------------------------------------------------------------------

def main( ):
    """
    The main function.

    @author: Marco Wegner
    """

    parser = OptionParser(usage="usage: %prog [options]", conflict_handler="resolve")
    parser.add_options(support.get_global_options( ))
    parser.add_options(_get_local_options( ))

    (options, args) = parser.parse_args( )

    num_args = len(args)

    # there are arguments present - but we cannot do anything with them
    if num_args > 0:
        support.print_warning_message("unused argument(s) %s" % (" ".join(args)))

    # the script has been called without any options (but perhaps with
    # ususable arguments)
    if (len(sys.argv) - num_args) == 1:
        parser.print_help( )
        sys.exit( )

    delimiters = support.extract_delimiters(options)

    # check if the input file is specified
    if options.infile is None:
        support.print_error_message("no input file specified")
        sys.exit( )

    pofile = Ut3PoFile(options.infile)
    store = pofile.parse( )

    if (options.verbose):
        print "\nProcessing %s:" % (os.path.basename(options.infile))
        po_count = len(store.get_all( ))
        po_untr = len(store.get_unfinished( ))
        print " * %i messages (%i translated, %i untranslated)" \
            % (po_count, po_count-po_untr,po_untr)

    # check if the template file is defined
    if options.template is None:
        support.print_error_message("no template file specified")
        sys.exit( )

    oname = "output"
    if options.outfile is not None:
        oname = os.path.basename(options.outfile)

    if options.verbose:
        print "\nAssembling %s:" % (oname)

    tmpl = UT3Template(options.template, delimiters)
    tmpl_count, tmpl_untr = tmpl.convert(store, options.outfile)

    if options.verbose:
        print " * %i messages (%i translated, %i untranslated)" \
            % (tmpl_count, tmpl_count-tmpl_untr, tmpl_untr)
        print "\n" + oname + " written."
        print "\nDone."

    return


# -----------------------------------------------------------------------------
# Helper functions
# -----------------------------------------------------------------------------

def _get_local_options( ):
    """
    Returns a list of options which are only used in this script.

    @return: The option list.
    """

    option_list = [
       make_option("-t", "--template", dest="template", help="specify the template file"),
       make_option("-i", "--input", dest="infile", help="specify the input file"),
       make_option("-o", "--output", dest="outfile", help="specify the output file")
    ]

    return option_list

# -----------------------------------------------------------------------------

if __name__ == "__main__":
    main( )



#  $Id: pofile.py 39 2007-08-06 10:48:59Z ogre.crewman $
#
#  This file is part of the UT3 project
#
#  Copyright (C) 2007 Marco Wegner <ogre.crewman@googlemail.com>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; see the file COPYING.  If not, write to
#  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#  Boston, MA 02110-1301, USA.


"""
This module contains classes and functions for dealing with PO and POT files.
"""

__version__ = "$Revision: 39 $"


# -----------------------------------------------------------------------------
# Import declarations
# -----------------------------------------------------------------------------

# modules
import logging, os, re, sys
from ut3 import store
# classes
from ut3.store import UT3Item, UT3Store
# functions
from time import gmtime, strftime
from ut3.support import print_error_message


# -----------------------------------------------------------------------------
# Global constants
# -----------------------------------------------------------------------------

LINE_UNKNOWN = -1
LINE_EMPTY   =  0
LINE_COMMENT =  1
LINE_MSGID   =  2
LINE_MSGSTR  =  3


# -----------------------------------------------------------------------------
# Classes
# -----------------------------------------------------------------------------

class Ut3PoFile:
    """
    This class represents a PO or POT file.

    @author: Marco Wegner
    """


    # -------------------------------------------------------------------------
    # Public methods
    # -------------------------------------------------------------------------

    def __init__(self, name):
        """
        Creates a new PO or POT file instance.
        """

        self.__name = name

        return

    # -------------------------------------------------------------------------

    def parse(self):
        """
        Parse the PO file.

        @return: The UT3 translation store filled with the parsed entries.
        """

        lines = self._get_file_lines( )

        message, translation     = None, None
        comment, status, sources = None, None, None
        temp_msgid, temp_msgstr  = None, None
        previous, current        = None, None

        tstore = UT3Store( )

        for curl in lines:

            curl = curl.strip( )

            if len(curl) == 0:
                current = LINE_EMPTY
            elif curl[0] == '#':
                current = LINE_COMMENT
                status, comment, sources = \
                    _extract_comment_values(curl, status, comment, sources)
            else:
                keyword, message = _parse_line(curl)
                if keyword is not None:
                    if keyword == "msgid":
                        current = LINE_MSGID
                        # and now initialise them for later use
                        temp_msgid = message
                        temp_msgstr = ""
                    elif keyword == "msgstr":
                        current = LINE_MSGSTR
                        temp_msgstr = message
                    else:
                        current = LINE_UNKNOWN
                        logging.error("unknown keyword: %s" % (keyword))
                else:
                    if message is not None:
                        if current == LINE_MSGID:
                            temp_msgid = temp_msgid + message
                        elif current == LINE_MSGSTR:
                            temp_msgstr = temp_msgstr + message
                        else:
                            logging.error("unknown mode")

            if previous == LINE_MSGSTR and current != LINE_MSGSTR:
                # we're not in msgstr mode anymore --> save the current entry
                entry = _make_item(message, translation, \
                    sources, comment, status)
                if entry is not None:
                    tstore.append(entry)

                # reset the item values
                message, translation     = None, None
                comment, status, sources = None, None, None

            # save msgid and msgstr for storing them later
            message = temp_msgid
            translation = temp_msgstr
            # save line state
            previous = current

        # finally append the last pair
        if previous == LINE_MSGSTR:
            entry = _make_item(message, translation, sources, comment, status)
            if entry is not None:
                tstore.append(entry)

        return tstore

    # -------------------------------------------------------------------------

    def write(self, tstore, pot_mode = False):
        """
        Actually writes the POT file.

        @param tstore: The UT3 translation store.
        @param pot_mode: If true then write a POT file.
        """

        if not isinstance(tstore, UT3Store):
            raise TypeError

        if self.__name is not None:
            # check if the file is writable
            if not os.access(os.path.dirname(self.__name), os.W_OK):
                print_error_message("PO file not writable")
                sys.exit( )
            out = file(self.__name, 'w')
        else:
            out = sys.stdout

        self._write_header(out, pot_mode)

        for ent in tstore.get_all( ):
            out.write('\n')

            if ent.comment is not None:
                out.write('#. %s\n' % (ent.comment))

            sources = ent.get_sources( )
            if len(sources) > 1 or (len(sources) == 1 and not "" in sources):
                out.write('#: %s\n' % (", ".join(sources)))

            if ent.translation is not None and ent.translation != "" and ent.status == store.STATUS_UNFINISHED:
                out.write('#, fuzzy\n')

            out.write('msgid \"%s\"\n' % (ent.message.replace('\"', '\\"')))
            tr = ""
            if ent.translation is not None:
                tr = ent.translation.replace('\"', '\\"')
            out.write('msgstr \"%s\"\n' % (tr.encode("utf-8")))

        if not self.__name is None:
            out.close( )

        return

    # -------------------------------------------------------------------------

    def _get_file_lines(self):
        """
        Extract the lines of the PO file and return it as string list.

        @return: The file's lines as string list
        """

        # check if the PO/POT file is readable
        if self.__name is None or not os.access(self.__name, os.R_OK):
            print_error_message("PO file does not exist or is not readable")
            sys.exit( )

        # read the PO file
        pofile = file(self.__name, 'r')
        lines = pofile.readlines( )
        pofile.close( )

        return lines

    # -------------------------------------------------------------------------

    def _write_header(self, out, pot_mode = False):
        """
        Write a PO or POT file's header.

        @param out: The handle to the file.
        @param pot_mode: If true then write a POT file.
        """

        if pot_mode:
            out.write('# This template has been generated by template2pot.py which is\n')
            out.write('# part of the Universal Template Translation Tools (UT3)\n')
            out.write('#\n')
            out.write('#, fuzzy\n')

        out.write('msgid \"\"\n')
        out.write('msgstr \"\"\n')

        package = 'PACKAGE VERSION'
        if not (pot_mode or self.__name is None):
            pname = os.path.basename(self.__name)
            wlist = re.split('\.po[t]?$', pname)
            if len(wlist) > 0 and len(wlist[0]) > 0:
                package = wlist[0]
        out.write('\"Project-Id-Version: %s\\n\"\n' % (package))

        out.write('\"Report-Msgid-Bugs-To: \\n\"\n')

        dstr = strftime('%Y-%m-%d %H:%M+0000', gmtime( ))
        if pot_mode:
            out.write('\"POT-Creation-Date: %s\\n\"\n' % (dstr))
            out.write('\"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\\n\"\n')
        else:
            out.write('\"POT-Creation-Date: YEAR-MO-DA HO:MI+ZONE\\n\"\n')
            out.write('\"PO-Revision-Date: %s\\n\"\n' % (dstr))

        out.write('\"Last-Translator: FULL NAME <EMAIL@ADDRESS>\\n\"\n')
        out.write('\"Language-Team: LANGUAGE <LL@li.org>\\n\"\n')
        out.write('\"MIME-Version: 1.0\\n\"\n')

        charset = 'CHARSET'
        if not pot_mode:
            charset = 'UTF-8'
        out.write('\"Content-Type: text/plain; charset=%s\\n\"\n' % (charset))

        out.write('\"Content-Transfer-Encoding: 8bit\\n\"\n')

        return


# -----------------------------------------------------------------------------
# Helper functions
# -----------------------------------------------------------------------------

def _parse_line(line):
    """
    Parses a single (non-comment and non-empty) line from the PO file and
    returns the keyword (if any) and message text for this line.

    @param line: The line to be parsed.
    @return: The extracted values for the keyword (or None for no keyword)
        and message text (or None in case of comments or empty lines)
        for the current line.
    """

    line = line.strip( )

    keyword = None

    spos = line.find('\"')
    if spos > 0:
        # the quote character is preceded by a keyword here
        pos = line.find(' ')
        keyword = line[0:pos]

    message = line[spos+1:-1]

    return keyword, message

# -----------------------------------------------------------------------------

def _extract_comment_values(line, status, comment, sources):
    """
    Extract the comments from the current line but change only the relevant 
    variable and keep the others.
    
    @param line: The current line.
    @param status: The current value of the translation status.
    @param comment: The current value of the translation comment.
    @param sources: The current value of the string sources.
    @return: The new values for status, comment and sources.
    """
    
    line = line.strip( )

    if len(line) > 1:
        if line[1] == ':':
            sources = re.split(',[\s]*', line[2:].strip( ))
        elif line[1] == '.':
            comment = line[2:].strip( )
        elif line[1] == ',':
            flags = re.split(',[\s]*', line[2:].strip( ))
            if not 'fuzzy' in flags:
                status = store.STATUS_COMPLETE
            
    return status, comment, sources

# -----------------------------------------------------------------------------

def _make_item(message, translation, sources, comment, status):
    """
    Creates a single UT3 translation item from a parsed entry in a PO file.

    @param message: The original (translatable) string.
    @param translation: The translation of the original string.
    @param sources: The source files (references) for this string.
    @param comment: An optional comment (or hint).
    @param status: The status of the translation (complete or unfinished).
    @return: The generated UT3 translation entry or None if the translatable
        string is empty or even None.
    """

    entry = None
    if (message is not None) and (message != "") and (translation is not None):
        # both the message and its translation are escaped in the PO file;
        # we need to undo this before storing the strings in the
        # translation entry
        entry = UT3Item(message.replace('\\"', '\"'))
        entry.translation = translation.replace('\\"', '\"')
        entry.extend_sources(sources)
        entry.comment = comment
        if status is None:
            status = store.STATUS_UNFINISHED
            if translation != "":
                status = store.STATUS_COMPLETE
        entry.status = status

    return entry
